module SVGSprite
  class Exporter
    XMLNS = {
      'xmlns' => 'http://www.w3.org/2000/svg',
      'xmlns:sketch' => 'http://www.bohemiancoding.com/sketch/ns',
      'xmlns:xlink' => 'http://www.w3.org/1999/xlink'
    }

    # Class methods
    def self.export!(svgs, options)
      exporter = self.new(svgs, options)
      exporter.create_output_dir!

      if options[:stdout]
        exporter.export_to_stdout
      else
        exporter.export_to_file
      end
    end

    # Instance methods
    def initialize(svgs, options)
      @options = options
      @svgs = svgs
    end

    def create_output_dir!
      FileUtils.mkdir_p(@options[:output])
    end

    def export_to_stdout
      STDOUT.write(svg_string)
    end

    def export_to_file
      output_file = "#{@options[:output]}/#{@options[:filename]}.svg"
      File.open(output_file, 'w') { |f| f.write(svg_string) }
    end

    def svg_string
      icon_nodes = @svgs.map { |svg_hash| icon_node(svg_hash) }
      xmlns = XMLNS.map { |k, v| %(#{k}="#{v}") }.join(' ')

      %(<svg style="display: none" #{xmlns}><defs>#{icon_nodes.join('')}</defs></svg>)
    end

    def icon_node(svg_hash)
      prefix = @options[:prefix]
      svg = svg_hash[:svg]
      filename = svg_hash[:filename]
      id = "#{prefix}-#{filename}"

      %(<g id="#{id}">#{svg}</g>)
    end
  end
end
